//
// DropSco - DragView.m
// These instance methods handle drag and drop of CMIX score files
//
//_____________________________________________________________________________________________________

#import "DragView.h"
#import "AppDelegate.h"
 
@implementation DragView

//_____________________________________________________________________________________________________ 
- (id)initWithFrame:(NSRect)frame{
    self = [super initWithFrame:frame];
    if (self) {
        [self registerForDraggedTypes:[NSArray arrayWithObject:NSFilenamesPboardType]];
    }
    return self;
}
//_____________________________________________________________________________________________________ 
- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender{
    highlight=YES;
    [self setNeedsDisplay: YES];
    return NSDragOperationGeneric;
}
//_____________________________________________________________________________________________________ 
- (void)draggingExited:(id <NSDraggingInfo>)sender{
    highlight=NO;
    [self setNeedsDisplay: YES];
}
//_____________________________________________________________________________________________________
CGMutablePathRef createRoundedCornerPath(CGRect rect, CGFloat cornerRadius) {
    
    // create a mutable path
    CGMutablePathRef path = CGPathCreateMutable();
    
    // get the 4 corners of the rect
    CGPoint topLeft = CGPointMake(rect.origin.x, rect.origin.y);
    CGPoint topRight = CGPointMake(rect.origin.x + rect.size.width, rect.origin.y);
    CGPoint bottomRight = CGPointMake(rect.origin.x + rect.size.width, rect.origin.y + rect.size.height);
    CGPoint bottomLeft = CGPointMake(rect.origin.x, rect.origin.y + rect.size.height);
    
    // move to top left
    CGPathMoveToPoint(path, NULL, topLeft.x + cornerRadius, topLeft.y);
    
    // add top line
    CGPathAddLineToPoint(path, NULL, topRight.x - cornerRadius, topRight.y);
    
    // add top right curve
    CGPathAddQuadCurveToPoint(path, NULL, topRight.x, topRight.y, topRight.x, topRight.y + cornerRadius);
    
    // add right line
    CGPathAddLineToPoint(path, NULL, bottomRight.x, bottomRight.y - cornerRadius);
    
    // add bottom right curve
    CGPathAddQuadCurveToPoint(path, NULL, bottomRight.x, bottomRight.y, bottomRight.x - cornerRadius, bottomRight.y);
    
    // add bottom line
    CGPathAddLineToPoint(path, NULL, bottomLeft.x + cornerRadius, bottomLeft.y);
    
    // add bottom left curve
    CGPathAddQuadCurveToPoint(path, NULL, bottomLeft.x, bottomLeft.y, bottomLeft.x, bottomLeft.y - cornerRadius);
    
    // add left line
    CGPathAddLineToPoint(path, NULL, topLeft.x, topLeft.y + cornerRadius);
    
    // add top left curve
    CGPathAddQuadCurveToPoint(path, NULL, topLeft.x, topLeft.y, topLeft.x + cornerRadius, topLeft.y);
    
    // return the path
    return path;
}
//_____________________________________________________________________________________________________
- (void)drawRect:(NSRect)rect
{

    super.layer.backgroundColor = CGColorCreateGenericRGB(0, 0, 0, 0.9);
    [super drawRect:rect];
    CGFloat w = 4.;
    
    if ( highlight ) {

        NSRect newRect = NSMakeRect(rect.origin.x+w/2, rect.origin.y+w/2, rect.size.width-w, rect.size.height-w);
        NSBezierPath *path;
        [NSBezierPath setDefaultLineWidth: w];
        path = [NSBezierPath bezierPathWithRoundedRect:newRect
                                               xRadius:w
                                               yRadius:w];
        [[NSColor orangeColor] set];

        [path stroke];
    }
}
//_____________________________________________________________________________________________________
// Although not needed in OSX 10.7, this was included for source compatibility with OSX 10.10
//_____________________________________________________________________________________________________
- (NSDragOperation)draggingSession:(NSDraggingSession *)session sourceOperationMaskForDraggingContext:(NSDraggingContext)context
{
    /*------------------------------------------------------
     NSDraggingSource protocol method.  Returns the types of operations allowed in a certain context.
     --------------------------------------------------------*/
    switch (context) {
        case NSDraggingContextOutsideApplication:
            return NSDragOperationCopy;
            
            //by using this fall through pattern, we will remain compatible
            // if the contexts get more precise in the future.
        case NSDraggingContextWithinApplication:
        default:
            return NSDragOperationCopy;
            break;
    }
}
//_____________________________________________________________________________________________________
- (BOOL)prepareForDragOperation:(id <NSDraggingInfo>)sender {
    highlight=NO;
    [self setNeedsDisplay: YES];
    return YES;
}
//_____________________________________________________________________________________________________ 
- (BOOL)performDragOperation:(id < NSDraggingInfo >)sender {
    NSArray *draggedFilenames = [[sender draggingPasteboard] propertyListForType:NSFilenamesPboardType];    
    NSURL* fileURL;    
    NSTextField* pathField = [(AppDelegate*)[[NSApplication sharedApplication] delegate] pathField];
    AppDelegate *delegate = (AppDelegate*) [NSApplication sharedApplication].delegate;
    const char* score = delegate->score;

    if (([[[draggedFilenames objectAtIndex:0] pathExtension] isEqual:@"txt"])||
    ([[[draggedFilenames objectAtIndex:0] pathExtension] isEqual:@"text"])||
    ([[[draggedFilenames objectAtIndex:0] pathExtension] isEqual:@"sco"])||
    ([[[draggedFilenames objectAtIndex:0] pathExtension] isEqual:@"cmix"])||
    ([[[draggedFilenames objectAtIndex:0] pathExtension] isEqual:@"score"])){
        fileURL=[NSURL URLFromPasteboard: [sender draggingPasteboard]];
        [pathField setStringValue: [fileURL path]];
        if([[fileURL path] getCString: (char*)score maxLength:512 encoding:NSUTF8StringEncoding] == YES)NSLog(@"%s", score);
        NSImage *image = [[[NSWorkspace sharedWorkspace] iconForFile:[[[sender draggingPasteboard] propertyListForType:NSFilenamesPboardType]objectAtIndex:0]]retain];
        [self setImage:image];
        
        return YES;
    } else return NO;
}
//_____________________________________________________________________________________________________
- (void)concludeDragOperation:(id <NSDraggingInfo>)sender{
 /*   NSArray *draggedFilenames = [[sender draggingPasteboard] propertyListForType:NSFilenamesPboardType];
    NSString *textDataFile = [NSString stringWithContentsOfFile:[draggedFilenames objectAtIndex:0] encoding:NSUTF8StringEncoding error:nil];
 
    NSLog(@"%@", textDataFile);*/
    AppDelegate *delegate = (AppDelegate*) [NSApplication sharedApplication].delegate;
    [[delegate goButton] setEnabled:YES];
}
 

 
@end